﻿using System;
using System.Diagnostics;
using System.Windows;
using System.Windows.Navigation;
using System.Windows.Threading;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using System.Device.Location;
using System.Net.NetworkInformation;
using System.Net;
using System.IO;

namespace SeaVan
{
    public delegate void AllDataCompleteEvent();

    public partial class App : Application
    {

        #region Fields

        private bool phoneApplicationInitialized = false;
        private static readonly object lockObject = new object();

        // The refresh interval is: canada ~5min, usa ~2min.
        private const double TIMER_INTERVAL = 180;
        private const string CANADA_URL = "http://apps.cbp.gov/bwt/bwt.xml";
        private const string USA_URL = "http://wsdot.wa.gov/traffic/rssfeeds/CanadianBorderTrafficData/Default.aspx";

        internal const string ERROR_MESSAGE_LOCATION_RACE = "Cannot get your start location - please try again in a little while";
        internal const string ERROR_MESSAGE_NETWORK = "Cannot get border data - please check your network connection, and try again later";
        internal const string ERROR_MESSAGE_DATA_CA = "Cannot get Canadian border data - please try again in a little while";
        internal const string ERROR_MESSAGE_DATA_US = "Cannot get US border data - please try again in a little while";
        internal const string ERROR_MESSAGE_LOCATION = "Your last known location is stale - directions will be approximate";
        internal const string ERROR_MESSAGE_GENERAL = "Sorry, we've hit a problem - please restart the app";
        internal const string ERROR_MESSAGE_DIRECTIONS = "Cannot get directions at the moment - please try again in a little while";
        internal const string ERROR_MESSAGE_MARKETPLACE = "Cannot connect to marketplace at the moment - please try again later";

        private static GeoCoordinateWatcher geoWatcher;
        private const double GEOWATCHER_THRESHOLD = 1000;
        private const double GEOWATCHER_TIMEOUT = 10;
        private static bool isDataCompleteUs;
        private static bool isDataCompleteCa;

        public static event AllDataCompleteEvent AllDataComplete;

        #endregion


        #region Properties

        public PhoneApplicationFrame RootFrame { get; private set; }

        private static BorderCrossings crossings;
        public static BorderCrossings Crossings
        {
            get
            {
                lock (lockObject)
                {
                    if (crossings == null)
                    {
                        crossings = new BorderCrossings();
                    }
                }
                return crossings;
            }
        }

        private static AppSettings settings;
        public static AppSettings AppSettings
        {
            get
            {
                lock (lockObject)
                {
                    if (settings == null)
                    {
                        settings = new AppSettings();
                    }
                }
                return settings;
            }
        }

        private static DispatcherTimer timer;
        public static DispatcherTimer Timer
        {
            get 
            {
                lock (lockObject)
                {
                    if (timer == null)
                    {
                        timer = new DispatcherTimer();
                        timer.Interval = TimeSpan.FromSeconds(TIMER_INTERVAL);
                    }
                }
                return timer;
            }
        }
       
        private static GeoCoordinate lastKnownPosition;
        public static GeoCoordinate LastKnownPosition
        {
            get { return lastKnownPosition; }
        }

        private static bool isGeoWatcherRunning;
        public static bool IsGeoWatcherRunning
        {
            get { return isGeoWatcherRunning; }
        }

        #endregion


        #region GeoCoordinateWatcher

        public static void StartGeoWatcher()
        {
            if (!isGeoWatcherRunning)
            {
                if (AppSettings.LocationSetting)
                {
                    lock (lockObject)
                    {
                        if (geoWatcher == null)
                        {
                            geoWatcher = new GeoCoordinateWatcher(GeoPositionAccuracy.High);
                            geoWatcher.MovementThreshold = GEOWATCHER_THRESHOLD;
                        }
                        geoWatcher.PositionChanged += geoWatcher_PositionChanged;
                        geoWatcher.Start();
                        isGeoWatcherRunning =
                            (geoWatcher.Status == GeoPositionStatus.Initializing
                            || geoWatcher.Status == GeoPositionStatus.Ready);
                    }
                }
            }
        }

        private static void geoWatcher_PositionChanged(object sender, GeoPositionChangedEventArgs<GeoCoordinate> e)
        {
            if (e.Position != null && e.Position.Location != null)
            {
                lastKnownPosition = e.Position.Location;
            }
        }

        public static void StopGeoWatcher()
        {
            if (geoWatcher != null)
            {
                geoWatcher.PositionChanged -= geoWatcher_PositionChanged;
                geoWatcher.Stop();
            }
        }
                
        #endregion


        #region Phone application initialization

        private void InitializePhoneApplication()
        {
            if (phoneApplicationInitialized)
            {
                return;
            }

            RootFrame = new PhoneApplicationFrame();
            RootFrame.Navigated += CompleteInitializePhoneApplication;
            RootFrame.NavigationFailed += RootFrame_NavigationFailed;
            phoneApplicationInitialized = true;
        }

        private void CompleteInitializePhoneApplication(object sender, NavigationEventArgs e)
        {
            if (RootVisual != RootFrame)
            {
                RootVisual = RootFrame;
            }
            RootFrame.Navigated -= CompleteInitializePhoneApplication;
        }

        #endregion

        
        #region Lifetime events

        public App()
        {
            UnhandledException += Application_UnhandledException;

            //if (System.Diagnostics.Debugger.IsAttached)
            //{
            //    Application.Current.Host.Settings.EnableFrameRateCounter = true;
            //    //Application.Current.Host.Settings.EnableRedrawRegions = true;
            //    //Application.Current.Host.Settings.EnableCacheVisualization = true;
            //}

            InitializeComponent();
            InitializePhoneApplication();
        }

        private void Application_Launching(object sender, LaunchingEventArgs e)
        {
        }

        private void Application_Activated(object sender, ActivatedEventArgs e)
        {
        }

        private void Application_Deactivated(object sender, DeactivatedEventArgs e)
        {
        }

        private void Application_Closing(object sender, ClosingEventArgs e)
        {
        }

        #endregion


        #region Exceptions

        private void RootFrame_NavigationFailed(object sender, NavigationFailedEventArgs e)
        {
            if (Debugger.IsAttached)
            {
                Debugger.Break();
            }
            else
            {
                ShowErrorMessage(ERROR_MESSAGE_GENERAL);
                e.Handled = true;
            }
        }

        private void Application_UnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e)
        {
            if (Debugger.IsAttached)
            {
                Debugger.Break();
            }
            else
            {
                ShowErrorMessage(ERROR_MESSAGE_GENERAL);
                e.Handled = true;
            }
        }

        internal static void ShowErrorMessage(string message)
        {
            if (Debugger.IsAttached)
            {
                Debug.WriteLine(message);
            }
            else
            {
                if (Deployment.Current.Dispatcher.CheckAccess())
                {
                    MessageBox.Show(message);
                }
                else
                {
                    Deployment.Current.Dispatcher.BeginInvoke(() => MessageBox.Show(message));
                }
            }
        }

        #endregion


        #region FetchWaitTimes

        public static void FetchWaitTimes()
        {
            StartGeoWatcher();

            if (!NetworkInterface.GetIsNetworkAvailable())
            {
                ShowErrorMessage(ERROR_MESSAGE_NETWORK);
                return;
            }

            isDataCompleteUs = false;
            isDataCompleteCa = false;
            WebClient webClientUsa = new WebClient();
            webClientUsa.OpenReadCompleted += webClientUs_OpenReadCompleted;

            // We have to append a unique (unused) querystring to force the WebClient to re-fetch instead of using the cache.
            try
            {
                webClientUsa.OpenReadAsync(new Uri(USA_URL + "?" + Guid.NewGuid().ToString()));
            }
            catch (WebException)
            {
                ShowErrorMessage(ERROR_MESSAGE_DATA_US);
                isDataCompleteUs = true;
                if (isDataCompleteUs && isDataCompleteCa)
                {
                    AllDataComplete();
                }
            }

            WebClient webClientCa = new WebClient();
            webClientCa.OpenReadCompleted += webClientCa_OpenReadCompleted;

            try
            {
                webClientCa.OpenReadAsync(new Uri(CANADA_URL + "?" + Guid.NewGuid().ToString()));
            }
            catch (WebException)
            {
                ShowErrorMessage(ERROR_MESSAGE_DATA_CA);
                isDataCompleteCa = true;
                if (isDataCompleteUs && isDataCompleteCa)
                {
                    AllDataComplete();
                }
            }
        }

        #endregion


        #region webClientUs_OpenReadCompleted

        private static void webClientUs_OpenReadCompleted(object sender, OpenReadCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                ShowErrorMessage(ERROR_MESSAGE_DATA_US);
                return;
            }

            using (Stream result = e.Result)
            {
                try
                {
                    CrossingXmlParser.ParseXmlUs(result);
                }
                catch (Exception ex)
                {
                    Debug.WriteLine(String.Format("webClientUs_OpenReadCompleted - CrossingXmlParser.ParseXmlUs: {0}", ex.Message));
                    ShowErrorMessage(ERROR_MESSAGE_DATA_US);
                }
                finally
                {
                    isDataCompleteUs = true;
                    if (isDataCompleteUs && isDataCompleteCa)
                    {
                        AllDataComplete();
                    }
                }
            }
        }

        #endregion


        #region webClientCa_OpenReadCompleted

        private static void webClientCa_OpenReadCompleted(object sender, OpenReadCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                ShowErrorMessage(ERROR_MESSAGE_DATA_CA);
                return;
            }

            using (Stream result = e.Result)
            {
                try
                {
                    CrossingXmlParser.ParseXmlCa(result);
                }
                catch (Exception)
                {
                    ShowErrorMessage(ERROR_MESSAGE_DATA_CA);
                }
                finally
                {
                    isDataCompleteCa = true;
                    if (isDataCompleteUs && isDataCompleteCa)
                    {
                        AllDataComplete();
                    }
                }
            }
        }

        #endregion

    }
}