﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using Microsoft.Phone.Tasks;

namespace SeaVan
{
    public partial class MainPage : PhoneApplicationPage
    {

        #region Fields

        private const double BORDER_LATITUDE = 49.0022;
        private ProgressIndicator progress;

        #endregion


        #region Init

        public MainPage()
        {
            InitializeComponent();

            appBarRefresh = (ApplicationBarIconButton)ApplicationBar.Buttons[1];

            DataContext = App.Crossings;
            CrossingsList.ItemsSource = App.Crossings.Crossings;

            progress = new ProgressIndicator(); 
            progress.IsIndeterminate = true;
            progress.Text = "fetching latest data...";

            Loaded += MainPage_Loaded;
            App.AllDataComplete += new AllDataCompleteEvent(App_AllDataComplete);
        }

        private void App_AllDataComplete()
        {
            if (!App.AppSettings.AutoRefreshSetting)
            {
                appBarRefresh.IsEnabled = true;
            }
            progress.IsVisible = false;
        }

        // Loaded happens after OnNavigatedTo, which is after the first screen is shown,
        // so we do the expensive operation here to improve startup time. We also
        // unhook the event after we're done - because we only need to do this once at
        // startup - subsequently, we'll refresh during OnNavigatedTo.
        private void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            FetchWaitTimes();
            Loaded -= MainPage_Loaded;
        }

        private void FetchWaitTimes()
        {
            // Start the progress indicator here, and stop it when both async web calls, and the results parsing are complete.
            progress.IsVisible = true;
            App.FetchWaitTimes();
        }

        #endregion


        #region Navigation

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            SystemTray.ProgressIndicator = progress;

            if (App.AppSettings.AutoRefreshSetting)
            {
                App.Timer.Tick += timer_Tick;
                App.Timer.Start();

                if (e.NavigationMode == NavigationMode.Back)
                {
                    FetchWaitTimes();
                }
            }

            appBarRefresh.IsEnabled = !App.AppSettings.AutoRefreshSetting;

            App.StartGeoWatcher();
        }

        protected override void OnNavigatedFrom(NavigationEventArgs e)
        {
            App.Timer.Tick -= timer_Tick;
            App.Timer.Stop();
            App.StopGeoWatcher();
        }

        private void timer_Tick(object sender, EventArgs e)
        {
            FetchWaitTimes();
        }

        #endregion


        #region AppBar Clicks

        private void appBarRefresh_Click(object sender, EventArgs e)
        {
            appBarRefresh.IsEnabled = false;
            FetchWaitTimes();
        }

        private void appBarSettings_Click(object sender, EventArgs e)
        {
            NavigationService.Navigate(new Uri("/SettingsPage.xaml", UriKind.Relative));
        }

        private void appBarAbout_Click(object sender, EventArgs e)
        {
            NavigationService.Navigate(new Uri("/AboutPage.xaml", UriKind.Relative));
        }

        #endregion


        #region CrossingsList_SelectionChanged

        // We need to use a GeoCoordinateWatcher to figure out which side of the border we're on,
        // so that we can tell Bing to give us directions without crossing the border.
        private void CrossingsList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (!App.AppSettings.LocationSetting || CrossingsList.SelectedIndex == -1)
            {
                CrossingsList.SelectedIndex = -1;
                return;
            }

            if (App.LastKnownPosition == null)
            {
                App.ShowErrorMessage(App.ERROR_MESSAGE_LOCATION_RACE);
                CrossingsList.SelectedIndex = -1;
                return;
            }

            if (!App.IsGeoWatcherRunning)
            {
                App.ShowErrorMessage(App.ERROR_MESSAGE_LOCATION);
            }

            BorderCrossing crossing = (BorderCrossing)CrossingsList.SelectedItem;
            BingMapsDirectionsTask directions = new BingMapsDirectionsTask();

            // If we're below the 49th parallel, our target endpoint is on the US side of the border,
            // otherwise its on the Canadian side. 
            if (App.LastKnownPosition.Latitude < BORDER_LATITUDE)
            {
                directions.End = new LabeledMapLocation(crossing.PortName, crossing.CoordinatesUs);
            }
            else
            {
                directions.End = new LabeledMapLocation(crossing.PortName, crossing.CoordinatesCa);
            }

            try
            {
                directions.Show();
            }
            catch
            {
                App.ShowErrorMessage(App.ERROR_MESSAGE_DIRECTIONS);
            }

            CrossingsList.SelectedIndex = -1;
        }

        #endregion

    }
}