//
//  SVContentController.m
//  SeaVan
//
//  Created by Andrew Whitechapel on 4/17/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import "SVContentController.h"
#import "SVViewController.h"
#import "SVBorderCrossing.h"
#import "constants.h"

static NSUInteger kNumberOfPages = 4;

@interface SVContentController (PrivateMethods)
- (void)loadScrollViewWithPage:(int)page;
- (void)scrollViewDidScroll:(UIScrollView *)sender;
@end

@implementation SVContentController

@synthesize scrollView, pageControl, viewControllers;

- (UIView *)view
{
    return self.scrollView;
}

- (void)awakeFromNib
{    
    // Initialize the model.
    [appDelegate refreshData];
    
    // Pages (that is, view controllers) are created lazily. In this intialization, 
    // we merely load the array with null placeholders which will be replaced later on demand.
    NSMutableArray *controllers = [[NSMutableArray alloc] init];
    for (unsigned i = 0; i < kNumberOfPages; i++)
    {
		[controllers addObject:[NSNull null]];
    }
    self.viewControllers = controllers;
    
    // A page is the width of the scroll view.
    scrollView.pagingEnabled = YES;
    scrollView.contentSize = 
        CGSizeMake(scrollView.frame.size.width * kNumberOfPages, scrollView.frame.size.height);
    scrollView.showsHorizontalScrollIndicator = NO;
    scrollView.showsVerticalScrollIndicator = NO;
    scrollView.scrollsToTop = NO;
    scrollView.delegate = self;
    
    pageControl.numberOfPages = kNumberOfPages;
    pageControl.currentPage = 0;
    
    // Pages are created on demand: load the visible page, plus the page on either
    // side to avoid flashes when the user starts scrolling.
    [self loadScrollViewWithPage:0];
    [self loadScrollViewWithPage:1];
}

- (void)loadScrollViewWithPage:(int)page
{
    if (page < 0)
    {
        return;
    }
    if (page >= kNumberOfPages)
    {
        return;
    }
        
    // Replace the null placeholder page, if necessary. We pass in a pointer to 
    // ourselves here so that the viewcontroller can call back in from the 
    // Refresh button Click to our refreshData method.
    SVViewController *controller = [viewControllers objectAtIndex:page];
    if ((NSNull *)controller == [NSNull null])
    {
        controller = 
            [[SVViewController alloc] initWithPageNumber:page parentController:self];
        [viewControllers replaceObjectAtIndex:page withObject:controller];
    }
    
    // Add the page's view to the scrollview.
    if (controller.view.superview == nil)
    {
        CGRect frame = scrollView.frame;
        frame.origin.x = frame.size.width * page;
        frame.origin.y = 0;
        controller.view.frame = frame;
        [scrollView addSubview:controller.view];
    }
}

- (void)updateViewFromData
{
    // Pull the data from the viewmodel to populate the UI.
    // This is our poor-man's data-binding.
    SVViewController *controller = [viewControllers objectAtIndex:pageControl.currentPage];
        
    // Last date/time the server data was updated.
    controller.lastUpdateSea.titleLabel.text = appDelegate.border.lastUpdateUs;
    controller.lastUpdateVan.titleLabel.text = appDelegate.border.lastUpdateCa;
    SVBorderCrossing *crossing = [appDelegate.border.crossings objectAtIndex:pageControl.currentPage];
    
    // Individual crossing and lane wait times.
    controller.titleLabel.text = crossing.portName;
    controller.stdUs.text = crossing.passengerDelayTimeUs;
    controller.nexUs.text = crossing.nexusDelayTimeUs;
    controller.rdyUs.text = crossing.readyDelayTimeUs;
    controller.trkUs.text = crossing.commercialDelayTimeUs;
    controller.fstUs.text = crossing.fastDelayTimeUs;
    controller.stdCa.text = crossing.passengerDelayTimeCa;
    controller.nexCa.text = crossing.nexusDelayTimeCa;
    controller.rdyCa.text = crossing.readyDelayTimeCa;
    controller.trkCa.text = crossing.commercialDelayTimeCa;
    controller.fstCa.text = crossing.fastDelayTimeCa;
}


// View mechanics - these are agnostic to viewmodel changes. ///////////////////

- (void)scrollViewDidScroll:(UIScrollView *)sender
{
    // We don't want a "feedback loop" between the UIPageControl and the scroll 
    // delegate in which a scroll event generated from the user hitting the page 
    // control triggers updates from the delegate method. We use a boolean to disable 
    // the delegate logic when the page control is used.
    if (pageControlUsed)
    {
        // Do nothing - the scroll was initiated from the page control, not the user dragging.
        return;
    }
	
    // Switch the indicator when more than 50% of the previous/next page is visible
    CGFloat pageWidth = scrollView.frame.size.width;
    int page = floor((scrollView.contentOffset.x - pageWidth / 2) / pageWidth) + 1;
    pageControl.currentPage = page;
    
    // Load the visible page and the page on either side of it (to avoid flashes 
    // when the user starts scrolling).
    [self loadScrollViewWithPage:page - 1];
    [self loadScrollViewWithPage:page];
    [self loadScrollViewWithPage:page + 1];
    
    // A possible optimization would be to unload the views+controllers which 
    // are no longer visible.
    
    [self updateViewFromData];
}

// At the beginning of scroll dragging, reset the boolean used when scrolls 
// originate from the UIPageControl.
- (void)scrollViewWillBeginDragging:(UIScrollView *)scrollView
{
    pageControlUsed = NO;
}

// At the end of scroll animation, reset the boolean used when scrolls originate 
// from the UIPageControl.
- (void)scrollViewDidEndDecelerating:(UIScrollView *)scrollView
{
    pageControlUsed = NO;
}

- (IBAction)changePage:(id)sender
{
    int page = pageControl.currentPage;
	
    // Load the visible page and the page on either side of it (to avoid flashes 
    // when the user starts scrolling).
    [self loadScrollViewWithPage:page - 1];
    [self loadScrollViewWithPage:page];
    [self loadScrollViewWithPage:page + 1];
    
	// update the scroll view to the appropriate page.
    CGRect frame = scrollView.frame;
    frame.origin.x = frame.size.width * page;
    frame.origin.y = 0;
    [scrollView scrollRectToVisible:frame animated:YES];
    
	// Set the flag used when scrolls originate from the UIPageControl - we're 
    // checking this in scrollViewDidScroll.
    pageControlUsed = YES;
    
    [self updateViewFromData];
}

@end
